// Microsoft Visual C++ generated resource script.
//
#include "../Shared/All.h"
#include "resource.h"

#define APSTUDIO_READONLY_SYMBOLS
/////////////////////////////////////////////////////////////////////////////
//
// Generated from the TEXTINCLUDE 2 resource.
//
#include "afxres.h"

/////////////////////////////////////////////////////////////////////////////
#undef APSTUDIO_READONLY_SYMBOLS

/////////////////////////////////////////////////////////////////////////////
// English (United States) resources

#if !defined(AFX_RESOURCE_DLL) || defined(AFX_TARG_ENU)
LANGUAGE LANG_ENGLISH, SUBLANG_ENGLISH_US
#pragma code_page(1252)

/////////////////////////////////////////////////////////////////////////////
//
// DESIGNINFO
//

#ifdef APSTUDIO_INVOKED
GUIDELINES DESIGNINFO
BEGIN
    IDD_WAV_INFO, DIALOG
    BEGIN
        LEFTMARGIN, 7
        RIGHTMARGIN, 195
        TOPMARGIN, 7
        BOTTOMMARGIN, 95
    END

    IDD_APE_INFO, DIALOG
    BEGIN
        LEFTMARGIN, 7
        RIGHTMARGIN, 242
        TOPMARGIN, 7
        BOTTOMMARGIN, 232
    END

    IDD_WINAMP_SETTINGS, DIALOG
    BEGIN
        LEFTMARGIN, 7
        RIGHTMARGIN, 322
        TOPMARGIN, 7
        BOTTOMMARGIN, 140
    END

    IDD_COMPRESSION, DIALOG
    BEGIN
        LEFTMARGIN, 4
        RIGHTMARGIN, 167
        TOPMARGIN, 4
        BOTTOMMARGIN, 72
    END
END
#endif    // APSTUDIO_INVOKED


#ifdef APSTUDIO_INVOKED
/////////////////////////////////////////////////////////////////////////////
//
// TEXTINCLUDE
//

1 TEXTINCLUDE 
BEGIN
    "resource.h\0"
END

2 TEXTINCLUDE 
BEGIN
    "#include ""afxres.h""\r\n"
    "\0"
END

3 TEXTINCLUDE 
BEGIN
    "\r\n"
    "\0"
END

#endif    // APSTUDIO_INVOKED


/////////////////////////////////////////////////////////////////////////////
//
// Icon
//

// Icon with lowest ID value placed first to ensure application icon
// remains consistent on all systems.
IDI_MONKEY              ICON                    "res\\MAC.ico"


/////////////////////////////////////////////////////////////////////////////
//
// Dialog
//

IDD_WINAMP_SETTINGS DIALOGEX 0, 0, 329, 147
STYLE DS_SYSMODAL | DS_SETFONT | DS_MODALFRAME | DS_CENTER | WS_POPUP | WS_CAPTION | WS_SYSMENU
CAPTION "Monkey's Audio Winamp Plugin Settings"
FONT 8, "MS Sans Serif", 0, 0, 0x0
BEGIN
    DEFPUSHBUTTON   "Save",IDOK,183,125,64,15
    PUSHBUTTON      "Cancel",IDCANCEL,258,125,64,15
    GROUPBOX        "Thread Priority",IDC_STATIC,6,3,171,42
    CONTROL         "Slider1",THREAD_PRIORITY_SLIDER,"msctls_trackbar32",TBS_AUTOTICKS | WS_TABSTOP,16,16,153,11
    LTEXT           "Lowest",IDC_STATIC,12,30,25,8
    CTEXT           "Normal",IDC_STATIC,72,30,40,8
    RTEXT           "Highest",IDC_STATIC,146,30,26,8
    GROUPBOX        "Options",IDC_STATIC,7,51,170,56
    CONTROL         "Ignore bitstream errors",IGNORE_BITSTREAM_ERRORS_CHECK,
                    "Button",BS_AUTOCHECKBOX | WS_TABSTOP,17,65,120,11
    CONTROL         "Suppress long passages of silence",SUPPRESS_SILENCE_CHECK,
                    "Button",BS_AUTOCHECKBOX | WS_TABSTOP,17,78,130,11
    CONTROL         "Scale output to 16-bit",SCALE_OUTPUT_CHECK,"Button",BS_AUTOCHECKBOX | WS_TABSTOP,17,92,130,8
    CTEXT           "(changes take effect on start of track)",IDC_STATIC,187,98,135,8
    GROUPBOX        "File Display Method",IDC_STATIC,183,3,140,89
    EDITTEXT        FILE_DISPLAY_METHOD_EDIT,193,18,120,12,ES_AUTOHSCROLL
    LTEXT           "%1 = ID3 Artist",IDC_STATIC,193,34,50,8
    LTEXT           "%2 = ID3 Title",IDC_STATIC,258,34,50,8
    LTEXT           "%3 = ID3 Album",IDC_STATIC,193,45,50,8
    LTEXT           "%7 = ID3 Track",IDC_STATIC,193,67,50,8
    LTEXT           "%5 = ID3 Comment",IDC_STATIC,193,56,65,8
    LTEXT           "%6 = ID3 Genre",IDC_STATIC,258,56,50,8
    LTEXT           "%8 = File Name",IDC_STATIC,258,67,50,8
    LTEXT           "%9 = Full Path",IDC_STATIC,193,78,53,8
    LTEXT           "%4 = ID3 Year",IDC_STATIC,258,45,47,8
    CONTROL         "",IDC_STATIC,"Static",SS_BLACKFRAME | SS_SUNKEN,7,117,315,1
END

IDD_COMPRESSION DIALOGEX 0, 0, 171, 76
STYLE DS_SETFONT | DS_MODALFRAME | WS_POPUP | WS_VISIBLE | WS_CAPTION | WS_SYSMENU
CAPTION "Monkey's Audio Compression Level"
FONT 8, "MS Sans Serif", 0, 0, 0x0
BEGIN
    CONTROL         "Fast",IDC_R1,"Button",BS_AUTORADIOBUTTON,16,12,62,10
    CONTROL         "Normal",IDC_R2,"Button",BS_AUTORADIOBUTTON,16,22,59,10
    CONTROL         "High",IDC_R3,"Button",BS_AUTORADIOBUTTON,16,32,57,10
    DEFPUSHBUTTON   "OK",IDOK,121,4,46,14
    PUSHBUTTON      "Cancel",IDCANCEL,121,22,46,14
    CONTROL         "",IDC_STATIC,"Static",SS_GRAYFRAME | SS_SUNKEN,4,4,98,65
    CONTROL         "Extra high",IDC_R4,"Button",BS_AUTORADIOBUTTON,16,42,57,10
    CONTROL         "Insane",IDC_R5,"Button",BS_AUTORADIOBUTTON,16,52,57,10
END

/////////////////////////////////////////////////////////////////////////////
//
// Version
//

VS_VERSION_INFO VERSIONINFO
 FILEVERSION APE_RESOURCE_VERSION_COMMA
 PRODUCTVERSION APE_RESOURCE_VERSION_COMMA
 FILEFLAGSMASK 0x3fL
#ifdef _DEBUG
 FILEFLAGS 0x1L
#else
 FILEFLAGS 0x0L
#endif
 FILEOS 0x40004L
 FILETYPE 0x1L
 FILESUBTYPE 0x0L
BEGIN
    BLOCK "StringFileInfo"
    BEGIN
        BLOCK "040904b0"
        BEGIN
            VALUE "Comments", "Compression and decompression library for Monkey's Audio"
            VALUE "CompanyName", "Matthew T. Ashland"
            VALUE "FileDescription", "Monkey's Audio DLL Library"
            VALUE "FileVersion", APE_RESOURCE_VERSION_STRING
            VALUE "LegalCopyright", APE_RESOURCE_COPYRIGHT
            VALUE "OriginalFilename", "MACDll.dll"
            VALUE "ProductName", "Monkey's Audio"
            VALUE "ProductVersion", APE_RESOURCE_VERSION_STRING
        END
    END
    BLOCK "VarFileInfo"
    BEGIN
        VALUE "Translation", 0x409, 1200
    END
END

#endif    // English (United States) resources
/////////////////////////////////////////////////////////////////////////////



#ifndef APSTUDIO_INVOKED
/////////////////////////////////////////////////////////////////////////////
//
// Generated from the TEXTINCLUDE 3 resource.
//


/////////////////////////////////////////////////////////////////////////////
#endif    // not APSTUDIO_INVOKED

