#pragma once

enum TB_BITMAPS
{
    TBB_NONE = -1,
    TBB_COMPRESS,
    TBB_DECOMPRESS,
    TBB_VERIFY,
    TBB_CONVERT,
    TBB_COMPRESSION_APE_FAST,
    TBB_COMPRESSION_APE_NORMAL,
    TBB_COMPRESSION_APE_HIGH,
    TBB_COMPRESSION_APE_EXTRA_HIGH,
    TBB_COMPRESSION_APE_INSANE,
    TBB_COMPRESSION_EXTERNAL,
    TBB_ADD_FILES,
    TBB_ADD_FOLDER,
    TBB_REMOVE_FILES,
    TBB_CLEAR_FILES,
    TBB_OPTIONS,
    TBB_STOP,
    TBB_PAUSE,
    TBB_MAKE_APL,
    TBB_OPTIONS_PROCESSING_GENERAL,
    TBB_OPTIONS_PROCESSING_VERIFY,
    TBB_OPTIONS_PROCESSING_ERROR_BEHAVIOR,
    TBB_OPTIONS_PROCESSING_OTHER,
    TBB_OPTIONS_OUTPUT_LOCATION,
    TBB_OPTIONS_OUTPUT_BEHAVIOR,
    TBB_OPTIONS_OUTPUT_OTHER,
    TBB_OPTIONS_PAGE_PROCESSING,
    TBB_OPTIONS_PAGE_OUTPUT,
    TBB_OPTIONS_PAGE_SIZE,
    TBB_COUNT
};
